/*
	utility program for the ProWesS loader

	DELETE : delete one or more files, or even entire directory trees.

	The parameters are all the files and or directories which have to be
	deleted, ans possibly some flags. No error messages are returned.

	possible flags :
	-tree : copy the entire subdirectory and create the subdirectories
		if they did not exist
	-progres, -progress : display progres report

	written by Joachim Van der Auwera for PROGS
	(c) 1996 by PROGS, Professional & Graphical Software
	started 24 April 1996
*/

#include "str.h"
#include "mem.h"
#include "dir.h"
#include "io.h"
#include "job.h"
#include "ProWesS.h"

#define MAX_MESSAGE IO_MAXFULLNAME

#define catch(x) do { if (err=(x)) return err; } while (0)

typedef struct {
	char **params;					/* parameters */
	char tree;						/* copy entire directory trees */
	char progres;					/* display progres of copying */
	Job subjob; 					/* subjob for progres report window */
	char display[MAX_MESSAGE];		/* progres report message */
	PWObject message;				/* object with message */
} Global;

Error delete(Global *g, char *filename);
Error progres(Global *g);

Error init(char *param)
{
	Error err;
	char **params;
	int count=0;					/* no of files to delete */
	Global g;

	MEMClear(sizeof(Global),&g);
	g.display[0]=' ';

	/* parse the command line parameters */
	catch( STRBreak(param,&g.params) );
	for (params=g.params ; *params ; params++)
	{
		if (STRSameCI("-path",*params))
		{
			if (!params[1]) return ERR_IPAR;
			*params++=NULL;
			*params  =NULL;
		}
		else
		if (STRSameCI("-tree",*params))
		{
			g.tree=TRUE;
			*params=NULL;
		}
		else
		if (STRSameCI("-progres",*params) || STRSameCI("-progress",*params))
		{
			g.progres=TRUE;
			*params=NULL;
		}
		else
			count++;
	}

	/* need something to delete */
	if (count==0) return ERR_IPAR;

	/* display progress report window */
	if (g.progres)
	{
		g.progres=FALSE;
		JOBCall(progres,"delete",64,DEFAULT_STACK,&g.subjob,1,&g);
	}

	/* params should indicate first file/directory to copy */
	params=g.params;
	while (!*params) params++;

	for ( ; count>0 ; params++, count--)
	{
		while (!*params) params++;
		delete(&g,*params);
	}
	return ERR_OK;
}

Error treedelete(Global *g, char *dirname)
{
	Error err;
	Channel in;
	DirIterator it;
	FileInfo info;
	char dir[IO_MAXDIRECTORY];
	char deltree=g->tree;

	catch( DIROpen(dirname,g->tree,&it,NULL) );
	catch( DIRDirectory(it,dir) );
	while (!DIRNext(it,&info))
	{
		STRCopySafe(dir,g->display,MAX_MESSAGE);
		STRAppendSafe(info.name,g->display,MAX_MESSAGE);
		if (g->progres) JOBEventSend(g->subjob,EVENT_LEVEL1);
		IODelete(g->display);
	}
	DIRClose(it);

	/* in case of tree, delete the directory subtree */
	/* this is done in a loop as some directories may still have */
	/* subdirectories in them when returned by DIRNext */
	while (deltree)
	{
		catch( DIRSubOpen(dirname,TRUE,&it) );
		catch( DIRSubDirectory(it,dir) );
		deltree=FALSE;
		while (!DIRSubNext(it,&info))
		{
			deltree=TRUE;
			STRCopySafe(dir,g->display,MAX_MESSAGE);
			STRAppendSafe(info.name,g->display,MAX_MESSAGE);
			if (g->progres) JOBEventSend(g->subjob,EVENT_LEVEL1);
			IODelete(g->display);
		}
		DIRSubClose(it);
	}

	return err;
}

Error delete(Global *g, char *filename)
{
	Channel file;
	FileType type;
	Error err;

	/* open the input and output files */
	catch( IOOpen(filename, OPEN_READ, &file) );
	catch( IOFileTypeGet(file,&type) );
	IOClose(file);
	if (type==FILETYPE_DIRECTORY)
		return treedelete(g,filename);
	else
	{
		if (g->progres)
		{
			STRCopySafe(filename,g->display,MAX_MESSAGE);
			JOBEventSend(g->subjob,EVENT_LEVEL1);
		}
		return IODelete(filename);
	}
}

Error redraw(PWObject object)
{
	Global *g;
	PWQuery(object,PW_GLOBAL_AUXILIARY,&g);
	return PWChange(g->message,PW_INFOSTRING_TEXT,g->display,NULL);
}

Error progres(Global *g)
{
	Error err;
	PWObject win;

	catch( PWCreate(NULL,&win,PW_TYPE_OUTLINE,NULL) );
	catch( PWCreate(win,NULL,PW_TYPE_INFOSTRING,
				PW_INFOSTRING_TEXT, "now deleting the file :              ",
				NULL) );
	catch( PWCreate(win,&g->message,PW_TYPE_INFOSTRING,
				PW_INFOSTRING_AUTOSIZE, FALSE,
				PW_INFOSTRING_TEXT, g->display,
				PW_GLOBAL_AUXILIARY, g,
				PW_SYSTEM_ACTION_EVENT1, redraw,
				NULL) );
	g->progres=TRUE;
	return PWActivate(win);
}
